<?php
namespace KnitPay\Gateways\Razorpay;

use Pronamic\WordPress\Pay\Plugin;
use Pronamic\WordPress\Pay\Payments\PaymentStatus;
use Razorpay\Api\Api;

/**
 * Title: Easebuzz Webhook Listner
 * Copyright: 2020 Knit Pay
 *
 * @author Knit Pay
 * @version 1.0.0
 * @since 2.2.4.0
 */
class Listener {


	public static function listen() {
		if ( ! filter_has_var( INPUT_GET, 'kp_razorpay_webhook' ) ) {
			return;
		}

		$post_body = file_get_contents( 'php://input' );
		$data      = json_decode( $post_body, true );

		if ( json_last_error() !== 0 ) {
			exit;
		}

		if ( empty( $data['event'] ) ) {
			exit;
		}

		$event_type = explode( '.', $data['event'] )[0];

		switch ( $event_type ) {
			case 'payment':
				$payment = get_pronamic_payment_by_transaction_id( $data['payload']['payment']['entity']['invoice_id'] );
				break;
			case 'subscription':
				if ( 'subscription.charged' !== $data['event'] ) {
					exit;
				}
				 $subscription_id = $data['payload']['subscription']['entity']['notes']['knitpay_subscription_id'];
				 $invoice_id      = $data['payload']['payment']['entity']['invoice_id'];
				 $payment_id      = $data['payload']['payment']['entity']['id'];

				if ( ! ( is_null( get_pronamic_payment_by_transaction_id( $invoice_id ) ) && is_null( get_pronamic_payment_by_transaction_id( $payment_id ) ) ) ) {
					exit;
				}

				 $subscription = \get_pronamic_subscription( $subscription_id );

				if ( ! isset( $subscription ) ) {
					exit;
				}

				if ( PaymentStatus::SUCCESS !== $subscription->get_first_payment()->get_status() ) {
					exit;
				}

				 $payment = pronamic_pay_plugin()->subscriptions_module->start_recurring( $subscription, Plugin::get_gateway( $subscription->config_id ), false );

				if ( isset( $subscription ) ) {
					$period = $subscription->new_period();
				}

				if ( isset( $period ) ) {
					$payment->add_period( $period );
				}

				 $payment->set_transaction_id( $invoice_id );
				 $payment->recurring = true;
				break;
			default:
				exit;
		}

		if ( null === $payment ) {
			exit;
		}

		// Add note.
		$note = sprintf(
		/* translators: %s: Razorpay */
			__( 'Webhook requested by %s.', 'knit-pay' ),
			__( 'Razorpay', 'knit-pay' )
		);

		$payment->add_note( $note );

		// Log webhook request.
		do_action( 'pronamic_pay_webhook_log_payment', $payment );

		// Update payment.
		Plugin::update_payment( $payment, false );
		exit;
	}
}
